<?php
/**
 * Oxygen Editor
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2021 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Editor
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Editor
 * @since 1.0.0
 */
final class Editor {

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		$this->register_elements();

		add_action( 'oxygen_add_plus_sections', [ $this, 'register_section' ] );
		add_action( 'oxygen_add_plus_wpgb_section_content', [ $this, 'register_subsection' ] );
		add_action( 'oxygen_enqueue_iframe_scripts', [ $this, 'dequeue_assets' ] );
		add_action( 'oxygen_enqueue_iframe_scripts', [ $this, 'preview_assets' ] );
		add_action( 'oxygen_enqueue_iframe_scripts', [ $this, 'localize_script' ] );

		add_filter( 'wp_grid_builder/frontend/enqueue_scripts', [ $this, 'unset_assets' ] );
		add_filter( 'wp_grid_builder/frontend/enqueue_styles', [ $this, 'unset_assets' ] );
		add_filter( 'wp_grid_builder/facet/choices', [ $this, 'add_choices' ], 10, 2 );
		add_filter( 'wp_grid_builder_caching/bypass', [ $this, 'bypass_cache' ] );
		add_filter( 'wp_grid_builder_map/has_facet', [ $this, 'has_facet' ] );

	}

	/**
	 * Register editor section
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function register_section() {

		\CT_Toolbar::oxygen_add_plus_accordion_section( 'wpgb', __( 'WP Grid Builder', 'wpgb-oxygen' ) );

	}

	/**
	 * Register editor subsection
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function register_subsection() {

		do_action( 'oxygen_add_plus_wpgb_basic' );

	}

	/**
	 * Register editor elements
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function register_elements() {

		new Elements\Grid();
		new Elements\Facet();

	}

	/**
	 * Dequeue WP Grid Builder assets in editor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function dequeue_assets() {

		foreach ( wpgb_get_styles() as $args ) {
			! empty( $args['handle'] ) && wp_dequeue_style( $args['handle'] );
		}

		foreach ( wpgb_get_scripts() as $args ) {
			! empty( $args['handle'] ) && wp_dequeue_script( $args['handle'] );
		}
	}

	/**
	 * Unset WP Grid Builder assets in preview mode to prevent conflict with grid shortcodes
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $assets Holds assets to enqueue.
	 * @return array
	 */
	public function unset_assets( $assets ) {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( isset( $_REQUEST['action'] ) && 'ct_render_shortcode' === $_REQUEST['action'] ) {
			$assets = [];
		}

		if ( defined( 'SHOW_CT_BUILDER' ) || defined( 'OXYGEN_IFRAME' ) ) {
			$assets = [];
		}

		return $assets;

	}

	/**
	 * Enqueue preview assets
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function preview_assets() {

		wp_enqueue_style(
			'wpgb-oxygen',
			WPGB_OXYGEN_URL . 'assets/css/preview.css',
			[],
			WPGB_OXYGEN_VERSION
		);

		wp_enqueue_script(
			'wpgb-oxygen',
			WPGB_OXYGEN_URL . 'assets/js/preview.js',
			[],
			WPGB_OXYGEN_VERSION,
			true
		);

	}

	/**
	 * Localize preview script
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function localize_script() {

		$post_types = get_post_types( [ 'public' => true ] );
		unset( $post_types['attachment'] );

		$data = array_merge(
			apply_filters( 'wp_grid_builder/frontend/localize_script', [] ),
			[
				'adminUrl'       => admin_url( 'admin.php' ),
				'frontStyles'    => wpgb_get_styles(),
				'frontScripts'   => wpgb_get_scripts(),
				'renderBlocks'   => (bool) wpgb_get_option( 'render_blocks' ),
				'templates'      => array_keys( apply_filters( 'wp_grid_builder/templates', [] ) ),
				'providers'      => array_keys( apply_filters( 'wp_grid_builder_oxygen/providers', [] ) ),
				'facetTypes'     => $this->facet_types(),
				'shadowGrids'    => [],
				'history'        => false,
				'hasGrids'       => true,
				'hasFacets'      => true,
				'hasLightbox'    => true,
				'noFacetContent' => esc_html__( 'No content found in facet.', 'wpgb-oxygen' ),
				'mainQuery'      => [
					'post_type'   => array_keys( $post_types ),
					'post_status' => 'publish',
				],
			]
		);

		wp_localize_script( 'wpgb-oxygen', 'wpgb_settings', $data );

	}

	/**
	 * Get facet types
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function facet_types() {

		global $wpdb;

		$facets = $wpdb->get_results( "SELECT id, type FROM {$wpdb->prefix}wpgb_facets" );

		return array_column( $facets, 'type', 'id' );

	}

	/**
	 * Add default choices in selection facet
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $choices Holds facet choices.
	 * @param array $facet   Holds facet settings.
	 * @return array
	 */
	public function add_choices( $choices, $facet ) {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ! isset( $_GET['wpgb-oxygen-builder'] ) ) {
			return $choices;
		}

		if ( empty( $facet['type'] ) || 'selection' !== $facet['type'] ) {
			return $choices;
		}

		return [
			(object) [
				'facet_value' => 'choice_1',
				'facet_name'  => 'Choice 1',
				'facet_slug'  => '_wpgb_oxy_selection',
			],
			(object) [
				'facet_value' => 'choice_2',
				'facet_name'  => 'Choice 2',
				'facet_slug'  => '_wpgb_oxy_selection',
			],
			(object) [
				'facet_value' => 'choice_3',
				'facet_name'  => 'Choice 3',
				'facet_slug'  => '_wpgb_oxy_selection',
			],
		];

	}

	/**
	 * Bypass cache when editing facets in editor
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param boolean $bypass Whether to bypass the cache.
	 * @return boolean
	 */
	public function bypass_cache( $bypass ) {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( isset( $_GET['wpgb-oxygen-builder'] ) ) {
			$bypass = true;
		}

		return $bypass;

	}

	/**
	 * Localize map facet assets in editor
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param boolean $has_facet Whether there is a map facet or not.
	 * @return boolean
	 */
	public function has_facet( $has_facet ) {

		if ( defined( 'SHOW_CT_BUILDER' ) ) {
			$has_facet = true;
		}

		return $has_facet;

	}
}
