<?php
/**
 * Override filter requests of WP Grid Builder
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2021 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Filter
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Filter
 * @since 1.0.0
 */
final class Filter {

	/**
	 * Requested template attributes
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $atts = [];

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		// We hook after caching add-on to allow caching.
		add_action( 'wp_grid_builder/async/render', [ $this, 'maybe_handle' ], 100 );
		add_action( 'wp_grid_builder/async/refresh', [ $this, 'maybe_handle' ], 100 );
		add_action( 'wp_grid_builder/async/search', [ $this, 'maybe_handle' ], 100 );

		// To correctly handle the element to be filtered asynchronously.
		add_filter( 'wp_grid_builder_oxygen/handle_element', [ $this, 'handle_element' ] );
		add_filter( 'wp_grid_builder_oxygen/provider_settings', [ $this, 'provider_settings' ] );

	}

	/**
	 * Check if it is an Oxygen element
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $atts Template attributes.
	 */
	public function maybe_handle( $atts ) {

		if ( ! isset( $atts['is_template'] ) || 'Oxygen' !== $atts['is_template'] ) {
			return;
		}

		$element = new Element( $atts );

		if ( empty( $element->content ) ) {
			return;
		}

		$this->atts = $atts;

		$content = ct_do_shortcode( $element->content );
		$action  = explode( '/', current_filter() );

		$this->{end( $action )}( $content, $atts );

	}

	/**
	 * Render facets on first load
	 *
	 * @since 1.0.0
	 * @access protected
	 *
	 * @param string $content Element content.
	 * @param array  $atts    Template attributes.
	 */
	protected function render( $content, $atts ) {

		wp_send_json(
			apply_filters(
				'wp_grid_builder/async/render_response',
				[
					'facets' => wpgb_refresh_facets( $atts ),
				],
				$atts
			)
		);

	}

	/**
	 * Refresh facets and content
	 *
	 * @since 1.0.0
	 * @access protected
	 *
	 * @param string $content Element content.
	 * @param array  $atts    Template attributes.
	 */
	protected function refresh( $content, $atts ) {

		wp_send_json(
			apply_filters(
				'wp_grid_builder/async/refresh_response',
				[
					'posts'  => $content,
					'facets' => wpgb_refresh_facets( $atts ),
				],
				$atts
			)
		);

	}

	/**
	 * Search for facet choices
	 *
	 * @since 1.0.0
	 * @access protected
	 *
	 * @param string $content Element content.
	 * @param array  $atts    Template attributes.
	 */
	protected function search( $content, $atts ) {

		wp_send_json(
			apply_filters(
				'wp_grid_builder/async/search_response',
				wpgb_search_facet_choices( $atts ),
				$atts
			)
		);

	}

	/**
	 * Whether to handle element
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param boolean $handle  Element Whether to handle element.
	 * @return boolean
	 */
	public function handle_element( $handle ) {

		if ( ! empty( $this->atts ) ) {
			$handle = true;
		}

		return $handle;

	}

	/**
	 * Set provider settings from Ajax request
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $settings Holds provider settings.
	 */
	public function provider_settings( $settings ) {

		if ( ! empty( $this->atts ) ) {
			$settings = $this->atts;
		}

		return $settings;

	}
}
