<?php
/**
 * Render elements
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2021 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Render
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Render
 * @since 1.0.0
 */
final class Render {

	use Helpers;

	/**
	 * Holds Oxygen elements to handle
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $elements = [];

	/**
	 * Whether inner content has been rendered
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $rendered = [];

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		add_filter( 'wp_grid_builder_oxygen/handle_element', [ $this, 'maybe_handle' ], 10, 2 );
		add_filter( 'wp_grid_builder/frontend/add_inline_script', [ $this, 'inline_script' ] );
		add_filter( 'wp_grid_builder/frontend/localize_script', [ $this, 'unset_elements' ] );
		add_action( 'ct_builder_start', [ $this, 'get_elements' ], 0 );

	}

	/**
	 * Whether to handle element
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param boolean $handle  Whether to handle element.
	 * @param integer $element Element name.
	 * @return boolean
	 */
	public function maybe_handle( $handle, $element ) {

		global $oxygen_vsb_css_caching_active;

		// We handle element when oxygen generates CSS.
		if ( $oxygen_vsb_css_caching_active ) {
			return true;
		}

		if ( ! isset( $this->elements[ $element ] ) ) {
			return false;
		}

		// Prevent rendering duplicate components from reusables.
		unset( $this->elements[ $element ] );

		return true;

	}

	/**
	 * Loop through elements to render
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $content Oxygen content.
	 */
	public function get_elements( $content = '' ) {

		if ( defined( 'SHOW_CT_BUILDER' ) || wpgb_doing_ajax() ) {
			return;
		}

		if ( empty( $content ) ) {
			$content = ct_template_shortcodes();
		}

		$pattern = $this->get_pattern( [ 'oxy-wpgb-facet', 'ct_inner_content', 'ct_reusable' ] );

		preg_match_all( '/' . $pattern . '/', $content, $matches, PREG_SET_ORDER );

		foreach ( $matches as $match ) {

			$tagname = preg_replace( '/_[0-9]*$/', '', $match[2] );

			$this->get_inner_content( $tagname, $match[3] );
			$this->get_reusables( $tagname, $match[3] );
			$this->get_element( $tagname, $match[3] );

		}
	}

	/**
	 * Get inner content
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $tagname Shortcode tagnem.
	 * @param array  $attr    Shortcode attributes.
	 */
	public function get_inner_content( $tagname, $attr ) {

		// To prevent infinite loop if nested inner content.
		if ( 'ct_inner_content' !== $tagname || $this->rendered ) {
			return;
		}

		$this->rendered = true;

		$content = get_post_meta( get_the_ID(), 'ct_builder_shortcodes', true );

		if ( empty( $content ) ) {
			return;
		}

		$this->get_elements( $content );

	}

	/**
	 * Get reusable elements
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $tagname Shortcode tagnem.
	 * @param array  $attr    Shortcode attributes.
	 */
	public function get_reusables( $tagname, $attr ) {

		if ( 'ct_reusable' !== $tagname ) {
			return;
		}

		$options = $this->get_options( $attr );

		if ( empty( $options['view_id'] ) ) {
			return;
		}

		$content = get_post_meta( $options['view_id'], 'ct_builder_shortcodes', true );

		if ( empty( $content ) ) {
			return;
		}

		$this->get_elements( $content );

	}

	/**
	 * Get and set element
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $tagname Shortcode tagnem.
	 * @param array  $attr    Shortcode attributes.
	 */
	public function get_element( $tagname, $attr ) {

		if ( 'oxy-wpgb-facet' !== $tagname ) {
			return;
		}

		$options = $this->get_options( $attr );

		if ( empty( $options['original'] ) ) {
			return;
		}

		$original = $options['original'];

		if ( empty( $original['oxy-wpgb-facet_grid'] ) ) {
			return;
		}

		$this->elements[ $original['oxy-wpgb-facet_grid'] ] = true;

	}

	/**
	 * Unset shadow elements generated by Oxygen facets
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $data Holds data to localize.
	 * @return array
	 */
	public function unset_elements( $data ) {

		if ( empty( $data['shadowGrids'] ) ) {
			return $data;
		}

		$data['shadowGrids'] = array_values(
			array_filter(
				$data['shadowGrids'],
				function( $id ) {
					return strpos( $id, 'oxygen-element-' ) === false;
				}
			)
		);

		return $data;

	}

	/**
	 * Add inline javascript code to instantiate Oxygen elements
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $script Inline script.
	 * @return string
	 */
	public function inline_script( $script ) {

		$rendered = [];

		foreach ( Container::all() as $id => $instance ) {
			$rendered[] = $instance->inline_script( $script );
		}

		if ( ! empty( $rendered ) ) {
			$script .= implode( '', array_unique( $rendered ) );
		}

		return apply_filters( 'wp_grid_builder/template/inline_script', $script );

	}
}
