<?php
/**
 * Base
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2021 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes\Providers;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Providers base
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Providers\Base
 * @since 1.0.0
 */
abstract class Base {

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $element  Holds element attributes.
	 * @param array $settings Holds Request settings.
	 */
	public function __construct( $element, $settings = [] ) {

		$this->element  = $element;
		$this->settings = $settings;

		add_filter( 'paginate_links', [ $this, 'paginate_links' ], PHP_INT_MAX );

	}

	/**
	 * Get element id
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function get_id() {

		return $this->element['id'];

	}

	/**
	 * Get post id
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function get_post_id() {

		return $this->settings['post_id'];

	}

	/**
	 * Get CSS selector
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function get_selector() {

		return '[data-options*="' . sanitize_html_class( $this->get_id() ) . '"]';

	}

	/**
	 * Filter element content
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $output Shortcode output.
	 * @return string
	 */
	public function render( $output ) {

		remove_filter( 'paginate_links', [ $this, 'paginate_links' ], PHP_INT_MAX );

		$output = preg_replace( '#(.?<\w+\s+\w+.*?)(>)#mi', "$1 {$this->inline_options()} $2", $output, 1 );
		$output = $this->after_render( $output );

		return $output;

	}

	/**
	 * Get element options to inline
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function inline_options() {

		$options = wp_json_encode(
			wp_parse_args(
				[
					'id'            => $this->get_id(),
					'postId'        => $this->get_post_id(),
					'isTemplate'    => 'Oxygen',
					'customContent' => true,
				],
				$this->options()
			)
		);

		return 'data-options="' . esc_attr( $options ) . '"';

	}

	/**
	 * Add inline javascript code to instantiate element
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $script Inline script.
	 * @return string
	 */
	public function inline_script( $script ) {

		$script .= 'window.addEventListener(\'wpgb.loaded\',function(){';
		$script .= 'var template=document.querySelector(' . wp_json_encode( $this->get_selector() ) . ');';
		$script .= 'if(template){';
		$script .= 'var wpgb=WP_Grid_Builder.instantiate(template);';
		$script .= 'wpgb.init&&wpgb.init();';
		$script .= $this->script();
		$script .= 'wpgb.facets&&wpgb.facets.on(\'appended\',function(p,m){var s=this.element.closest(\'.swiper-container\');(s=s&&s.swiper)&&(s.loopDestroy(),s.loopCreate(),s.update(),m===\'replace\'&&s.slideToLoop(0,0))})';
		$script .= '}});';

		return apply_filters( 'wp_grid_builder_oxygen/inline_script', $script );

	}

	/**
	 * Replace paginated links with current permalink.
	 *
	 * @since 1.0.0
	 *
	 * @param string $link The paginated link URL.
	 * @return string
	 */
	public function paginate_links( $link ) {

		$query_string = wpgb_get_query_string();

		if ( empty( $query_string ) && ! wpgb_doing_ajax() ) {
			return $link;
		}

		if ( ! empty( $this->settings['permalink'] ) ) {

			$link = str_replace(
				trailingslashit( home_url( '', 'relative' ) ),
				trailingslashit( wp_make_link_relative( $this->settings['permalink'] ) ),
				$link
			);
		}

		$keys = preg_filter( '/^/', '_', array_keys( $query_string ) );
		$keys = array_merge( [ 'wpgb-ajax', 'action' ], $keys );
		$link = str_replace( '#038;', '&', $link );
		$link = remove_query_arg( $keys, $link );

		return $link;

	}

	/**
	 * Get element options to inline
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function options();

	/**
	 * Get element script to localize
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function script();

	/**
	 * Before element render
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function before_render();

	/**
	 * Add inline script
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function after_render();
}
