<?php
/**
 * Provider
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2021 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes\Providers;

use WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Default provider
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Providers\Provider
 * @since 1.0.0
 */
class Provider extends Base {

	/**
	 * Get element options
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function options() {

		return [
			'itemSelector' => $this->get_selector() . ' > *',
			'isMainQuery'  => (
				empty( $this->element['original']['wp_query'] ) ||
				'default' === $this->element['original']['wp_query']
			),
		];

	}

	/**
	 * Get inline script
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function script() {

		$script  = 'var p=1;template.addEventListener(\'click\',function(e){';
		$script .= 'var t=e.target;p=(t.href||\'\').match(/\/(\d+)/);';
		$script .= '(t.classList.contains(\'page-numbers\')&&t.tagName===\'A\')&&(e.preventDefault()||wpgb.facets.refresh())});';
		$script .= 'wpgb.facets.on(\'fetch\',function(d){d.paged=(p&&p[1]||1);p=1});';

		return $script;

	}

	/**
	 * Apply filters for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function before_render() {

		Includes\add_action_once( 'pre_get_posts', [ $this, 'query_args' ] );
		Includes\add_action_once( 'loop_no_results', [ $this, 'no_results' ] );

		$this->wp_query();

	}

	/**
	 * Filter content output
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $content Element content.
	 * @return string
	 */
	public function after_render( $content = '' ) {

		if ( ! empty( $this->element['original']['wpgb_hide_pagination'] ) && method_exists( $this, 'remove_pagination' ) ) {
			$content = $this->remove_pagination( $content );
		}

		return $content;

	}

	/**
	 * Add/set arguments in query to allows filtering
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 */
	public function query_args( $query ) {

		$query->set( 'wp_grid_builder', $this->get_id() );

		if ( ! empty( $this->settings['lang'] ) ) {
			$query->set( 'lang', $this->settings['lang'] );
		}

		if ( ! empty( $this->settings['paged'] ) ) {

			$query->set( 'paged', (int) $this->settings['paged'] );
			set_query_var( 'paged', $query->get( 'paged' ) );

		}

		if ( ! empty( $this->element['original']['wpgb_hide_pagination'] ) ) {
			$query->set( 'no_found_rows', true );
		}
	}

	/**
	 * Run main WordPress query
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function wp_query() {

		global $wp_query;

		if ( ! wpgb_doing_ajax() ) {
			return;
		}

		if ( empty( $this->settings['main_query'] ) ) {
			return;
		}

		if (
			! empty( $this->element['original']['wp_query'] ) &&
			'default' !== $this->element['original']['wp_query']
		) {
			return;
		}

		// We override the main query when filtering.
		$wp_query = new \WP_Query( $this->settings['main_query'] ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited

	}

	/**
	 * Display no results message if no content found
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function no_results() {

		if ( empty( $this->element['original']['wpgb_noresults'] ) ) {
			return;
		}

		echo '<div>' . wp_kses_post( oxygen_vsb_base64_decode( $this->element['original']['wpgb_noresults'] ) ) . '</div>';

	}
}
